/*
 * Copyright (C) 2018 Nick Downing <nick@ndcode.org>
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

let BuildCache = require('@ndcode/build_cache')
let CleanCSS = require('@ndcode/clean-css')
let assert = require('assert')
let disk_build = require('@ndcode/disk_build')
let fs = require('fs')
let path = require('path')
let util = require('util')

let clean_css = new CleanCSS({returnPromise: true})
let fs_readFile = util.promisify(fs.readFile)
let fs_writeFile = util.promisify(fs.writeFile)

let MinCSSCache = function(diag) {
  if (!this instanceof MinCSSCache)
    throw Error('MinCSSCache is a constructor')
  this.diag = diag || false

  this.build_cache = new BuildCache()
}

MinCSSCache.prototype.get = function(key) {
  return /*await*/ this.build_cache.get(
    key,
    async result => {
      let render = await disk_build(
        key,
        async temp_pathname => {
          let render = await clean_css.minify(
            await fs_readFile(key, {encoding: 'utf-8'})
          )
          for (let i = 0; i < render.warnings.length; ++i)
            console.log(`clean-css warning: ${render.warnings[i]}`)
          return /*await*/ fs_writeFile(
            temp_pathname,
            render.styles,
            {encoding: 'utf-8'}
          )
        },
        this.diag
      )
      assert(render.deps.length === 0)
      result.value = await fs_readFile(render.pathname)
    }
  )
}

module.exports = MinCSSCache
